#NOTE: Paths are relative to the folder Code_Data/Scripts
combData <- read.table("../Feature_tables/combinedData.csv", sep =",", header = TRUE)
dim(combData)
head(combData)
combData2 <- combData[,-8]#remove IDs
head(combData2)

library(randomForest)
tuneRF(combData2[,-8], y = factor(combData2[,8]), ntreeTry = 400, mtryStart = 2)

# oscillates btwn 2 and 4. Let's use 2 as in Python classifier
set.seed(1234)
RF <- randomForest(x = combData2[,-8], y = factor(combData2[,8]), mtry = 2, ntree = 400, importance = TRUE)

#decrease in accuracy
varImp <- importance(RF, scale = TRUE)
varImpPlot(RF, sort = TRUE, type = 1)

ord <- rev(order(varImp[, 3], decreasing = TRUE)[1:7])
dotchart(varImp[ord, 3], xlab ="Mean Decrease in Accuracy", ylab = "", pch = 19,
         main = "", xlim = c(min(varImp[,3]), max(varImp[, 3])), 
         labels = rev(c("Distance Left ORF", "Distance Right ORF", "Distance Terminator", 
                    "Free Energy Secondary Structure", "Distance Promoter",
                    "Same Strand Right ORF", "Same Strand Left ORF")) )
varImpPlot(RF, sort = T, type = 1, main = "")

# RF is a random forests classifier
# data is the feature table
#labels is a vector with the names the instances IDs and as value 1s or 0s.
evaluateData <- function(RF, data, labels){
	require(ROCR)
	require(PRROC)
	require(randomForest)
	res <- list()
	#obtain predictions
	res$predD <- predict(RF, data, type = "prob")
	#Evaluate predictions
	res$pred <- prediction(res$predD[,2], as.logical(labels))
	res$PR <- performance(res$pred, measure = "prec", x.measure = "rec")
	res$SS <- performance(res$pred, measure="sens", x.measure="spec")
	res$auc  <- performance(res$pred, measure = "auc")
	res$acc <-  performance(res$pred, measure = "acc")
	res$pr <- pr.curve(scores.class0 = res$predD[labels == 1,2], scores.class1 = res$predD[labels == 0,2], curve = T)
    return(res)
}

#test with validation data
org <- c("E_coli", "M_tuberculosis", "R_capsulatus", "S_enterica", "S_pyogenes")
dir = "data/"
#For all validation data
resRF <- list()
for (i in org) {
  valData <- read.table(paste(dir,i,"_validation.csv", sep =""), sep = ",", header = TRUE)
  valLabels <- read.table(paste(dir,i,"_validation_labels.csv", sep = ""), sep = ",", header = TRUE, row.names = 1)
  valLabelsV <- valLabels[,1,drop = T]
  names(valLabelsV) <- row.names(valLabels)  
  print(head(valLabelsV))
  print(table(valLabels[,1]))
  #Obtain predictions
  resRF[[i]]<- evaluateData(RF, valData, valLabelsV)
}

### Lu data
LuP <- read.table("../Feature_tables/Lu_Positives.tsv", sep = "\t", header = T, row.names =1)
LuN <- read.table("../Feature_tables//Lu_Negatives.tsv", sep = "\t", header = T, row.names =1)
LuData <- rbind(LuP, LuN)
LuLabels <- vector(mode = "numeric", length = nrow(LuP) + nrow(LuN))
names(LuLabels) <- c(row.names(LuP), row.names(LuN))
LuLabels[row.names(LuP)] <- 1
resRF[["Lu"]] <- evaluateData(RF, LuData, LuLabels)

##S_enterica_LT2_data
LT2P <- read.table("../Feature_tables/SLT2_Positives.tsv", sep = "\t", header = T, row.names =1)
LT2N <- read.table("../Feature_tables/SLT2_Negatives.tsv", sep = "\t", header = T, row.names =1)
LT2Data <- rbind(LT2P, LT2N)
LT2Labels <- vector(mode = "numeric", length = nrow(LT2P) + nrow(LT2N))
names(LT2Labels) <- c(row.names(LT2P), row.names(LT2N))
LT2Labels[row.names(LT2P)] <- 1
resRF[["LT2"]] <- evaluateData(RF, LT2Data, LT2Labels)

#Check AUCs
sapply(resRF, function(i){i$auc})
## Generate PR curves
plot(resRF[["E_coli"]]$pr)
plot(resRF[["M_tuberculosis"]]$pr)
plot(resRF[["R_capsulatus"]]$pr)
plot(resRF[["S_enterica"]]$pr)
plot(resRF[["S_pyogenes"]]$pr)
plot(resRF[["Lu"]]$pr)
plot(resRF[["LT2"]]$pr)

##Generate Specificity/Sensitivity curve for SLT2 data.
cbind(slot(resRF[["LT2"]]$SS, "y.values")[[1]], slot(resRF[["LT2"]]$SS, "x.values")[[1]])
plot(resRF[["LT2"]]$SS, main = "SLT2 dataset", lwd = 2, col= "darkgreen", ylim = c(0,1))
points(0.67, 0.78, col = "red", pch = 19)
points(0.8511, 0.9158, col = "darkblue", pch = 19)
legend(0.1,0.4, c("sRNARanking", "Arnedo et al", "Barman et al"), fill = c("darkgreen", "red", "darkblue"))

#######################################################################
####### Statistical Analysis of differences in classifiers performance

createViolinPlot <- function(df, xVar, yVar, label= yVar){
  #Violin plots of Precision per Classifier 
  #TBD separated by recall
  require(ggplot2)
  # Basic violin plot
  p <- ggplot(df, aes_string(y=yVar, x=xVar)) +  geom_violin() + ylim(0,1) + ylab(label)
  p + geom_boxplot(width=0.1)
}


#################
#Read results (5 runs, 5 classifier, 4 train sets, 5 validation sets, 6 measures)
#NOTE: see README file for instructions about how to get the file performanceMetrics_r1_5.txt
results <- read.table("performanceMetrics_r1_5.txt", sep = "\t", header = FALSE, stringsAsFactors = F)
colnames(results) <- c("Classifier", "train", "test", "Measure", "Value")

results$train <- gsub("rcap", "Rc", results$train)
results$train <- gsub("sent", "Se", results$train)
results$train <- gsub("spyo", "Sp", results$train)
results$train <- gsub("all_data", "Combined", results$train)

meanR <- list()
sdR <- list()
medR <- list()

#Get results per model
for (i in unique(results$Measure)){
	tmp <- results[results$Measure == i,]
	f <- cbind(tmp$test, paste(tmp$Classifier,tmp$train, sep = "-"))
	colnames(f) <- c("test", "Classifier,train")	
	meanR[[i]] <- tapply(tmp[,"Value"], list(f[,2], f[,1]), mean)
	sdR[[i]] <- tapply(tmp[,"Value"], list(f[,2], f[,1]), sd)
	medR[[i]] <- tapply(tmp[,"Value"], list(f[,2], f[,1]), median)
}


#Violin plots of Precision per Classifier
createViolinPlot(results[results$Measure == "P10",], "Classifier", "Value", "Precision")
createViolinPlot(results[results$Measure == "P40",], "Classifier", "Value", "Precision")
createViolinPlot(results[results$Measure == "P60",], "Classifier", "Value", "Precision")
createViolinPlot(results[results$Measure == "AUPRC",], "Classifier", "Value", "AUPRC")
createViolinPlot(results[results$Measure == "AUC",], "Classifier", "Value", "AUC")

#Analysis of variance
av = aov(Value ~ Classifier * train, data = results[results$Measure == "AUPRC",])
summary(av)

require("PMCMR")
friedman.test(t(meanR$AUPRC))
posthoc.friedman.nemenyi.test(t(meanR$AUPRC))

############################
### Change to ranks without LR
library(plyr)
library(ggplot2)

rTable <- (apply(meanR$AUPRC[!grepl("LR", row.names(meanR$AUPRC)), ], 2, rank, ties.method = "max") -17) *-1

rTablePerClassifier <- cbind(colMeans(rTable[grepl("AB", row.names(rTable)),]), colMeans(rTable[grepl("GB", row.names(rTable)),]), colMeans(rTable[grepl("MP", row.names(rTable)),]), colMeans(rTable[grepl("RF", row.names(rTable)),]))

colnames(rTablePerClassifier) <- c("AB", "GB", "MP", "RF")


rankByModel <- t(apply(rTable, 1, function(r){
	Rank.mean = mean(r)
	Rank.sd = sd(r)
	Length = length(r)
	tfrac = qt(p=.9, df = Length -1)
	Lower = Rank.mean - tfrac * Rank.sd / sqrt (Length)
	Upper = Rank.mean +tfrac*Rank.sd/sqrt(Length)
	c(Rank.mean, Rank.sd, Length, tfrac, Lower, Upper)
}))

colnames(rankByModel) <- c("Rank.mean", "Rank.sd", "Length", "tfrac", "Lower", "Upper")
rankByModel_df <- as.data.frame(rankByModel)
rankByModel_df$model <- row.names(rankByModel)
rankByModel_df$Classifier <- gsub("-.*", "", row.names(rankByModel))
rankByModel_df$train <- gsub(".*-", "", row.names(rankByModel))

rankByClassifier <-  t(apply(rTablePerClassifier, 2, function(r){
	Rank.mean = mean(r)
	Rank.sd = sd(r)
	Length = length(r)
	tfrac = qt(p=.9, df = Length -1)
	Lower = Rank.mean - tfrac * Rank.sd / sqrt (Length)
	Upper = Rank.mean +tfrac*Rank.sd/sqrt(Length)
	c(Rank.mean, Rank.sd, Length, tfrac, Lower, Upper)
}))

colnames(rankByClassifier) <- c("Rank.mean", "Rank.sd", "Length", "tfrac", "Lower", "Upper")
rankByClassifier <- as.data.frame(rankByClassifier)
rankByClassifier$model <- row.names(rankByClassifier)
rankByClassifier$Classifier <- gsub("-.*", "", row.names(rankByClassifier))
rankByClassifier$train <- gsub(".*-", "", row.names(rankByClassifier))

rankByClassifier


AUPRCByModel <- as.data.frame(cbind(
	AUPRC.mean = rowMeans(meanR$AUPRC), 
	AUPRC.sd = rowMeans(sdR$AUPRC),
	Length = 5,
	tfrac = qt(p=.9, df = 4)))
AUPRCByModel <- AUPRCByModel[!grepl("LR", row.names(AUPRCByModel)),]
AUPRCByModel$Lower = AUPRCByModel$AUPRC.mean - AUPRCByModel$tfrac * AUPRCByModel$AUPRC.sd / sqrt (AUPRCByModel$Length)
AUPRCByModel$Upper = AUPRCByModel$AUPRC.mean + AUPRCByModel$tfrac * AUPRCByModel$AUPRC.sd / sqrt(AUPRCByModel$Length)
AUPRCByModel$model <- row.names(AUPRCByModel)
AUPRCByModel$Classifier <- gsub("-.*", "", row.names(AUPRCByModel))
AUPRCByModel$train <- gsub(".*-", "", row.names(AUPRCByModel))

##### ANOVA
av = aov(Value ~ Classifier * train, data = results[results$Measure == "AUPRC" & results$Classifier !="LR",])
summary(av)
TukeyHSD(av, which = c("Classifier","train"))

### Friedman test
require("PMCMR")
friedman.test(t(rTable)) # same as friedman.test(t(meanR$AUPRC[!grepl("LR", row.names(meanR$AUPRC)),]))
posthoc.friedman.nemenyi.test(t(rTable))
friedman.test(rTablePerClassifier)
posthoc.friedman.nemenyi.test(rTablePerClassifier)
posthoc.friedman.conover.test(y=rTablePerClassifier, p.adjust="fdr")

### quade test
quade.test(rTablePerClassifier)
posthoc.quade.test(rTablePerClassifier,  dist="TDist", p.adj="fdr")

### Wilcoxon test
with(results[results$Measure == "AUPRC" & results$Classifier !="LR",], pairwise.wilcox.test(Value, Classifier, "fdr", alternative = "l", paired = TRUE))
with(results[results$Measure == "AUPRC" & results$Classifier !="LR",], pairwise.wilcox.test(Value, Classifier, "fdr", alternative = "g", paired = TRUE))
######
# Generate CD plot
library("scmamp")
plotCD(rTablePerClassifier, alpha = 0.05)

